const bcrypt = require("bcryptjs");
const crypto = require("crypto");
// const { v4: uuidv4 } = require("uuid"); // UUIDs will be generated by PostgreSQL
const pool = require("../config/database"); // db is now the pg Pool
const logger = require("../../config/logger"); // Corrected logger path

class User {
	// No need for _run, _get, _all as pool.query returns a promise with a consistent result object.

	// Create a new user
	static async create(userData) {
		const {
			email,
			password,
			first_name,
			last_name,
			role = "user",
			is_verified = false, // PostgreSQL uses true/false for BOOLEAN
		} = userData;

		const saltRounds = 12;
		const password_hash = await bcrypt.hash(password, saltRounds);
		const verification_token = crypto.randomBytes(32).toString("hex");
		// UUID is generated by DB default (gen_random_uuid())

		const query = `
      INSERT INTO users (email, password_hash, first_name, last_name, role, is_verified, verification_token, created_at, updated_at)
      VALUES ($1, $2, $3, $4, $5, $6, $7, NOW(), NOW())
      RETURNING id, uuid, email, first_name, last_name, role, is_verified, verification_token;
    `;
		const values = [
			email,
			password_hash,
			first_name,
			last_name,
			role,
			is_verified,
			verification_token,
		];

		try {
			const result = await pool.query(query, values);
			return result.rows[0]; // Returns the newly created user data including id and uuid
		} catch (error) {
			// PostgreSQL error codes: https://www.postgresql.org/docs/current/errcodes-appendix.html
			if (error.code === "23505") {
				// unique_violation
				if (error.constraint === "users_email_key") {
					// Or whatever your unique constraint name for email is
					throw new Error("Email already exists");
				}
				// Potentially other unique constraints like users_uuid_key if not handled by default generation
			}
			logger.error("Error creating user:", error);
			throw error;
		}
	}

	// Find user by email
	static async findByEmail(email) {
		const query = "SELECT * FROM users WHERE email = $1 AND is_active = TRUE";
		const { rows } = await pool.query(query, [email]);
		return rows[0];
	}

	// Find user by ID
	static async findById(id) {
		const query = "SELECT * FROM users WHERE id = $1 AND is_active = TRUE";
		const { rows } = await pool.query(query, [id]);
		return rows[0];
	}

	// Find user by UUID
	static async findByUuid(uuid) {
		const query = "SELECT * FROM users WHERE uuid = $1 AND is_active = TRUE";
		const { rows } = await pool.query(query, [uuid]);
		return rows[0];
	}

	// Find user by verification token
	static async findByVerificationToken(token) {
		const query = "SELECT * FROM users WHERE verification_token = $1";
		const { rows } = await pool.query(query, [token]);
		return rows[0];
	}

	// Find user by password reset token
	static async findByPasswordResetToken(token) {
		const query = `
      SELECT * FROM users 
      WHERE password_reset_token = $1 
      AND password_reset_expires > NOW()
      AND is_active = TRUE
    `;
		const { rows } = await pool.query(query, [token]);
		return rows[0];
	}

	// Verify email
	static async verifyEmail(token) {
		const query = `
      UPDATE users 
      SET is_verified = TRUE, verification_token = NULL -- updated_at is handled by trigger
      WHERE verification_token = $1
      RETURNING id;
    `;
		const result = await pool.query(query, [token]);
		return result.rowCount > 0;
	}

	// Update password
	static async updatePassword(id, newPassword) {
		const saltRounds = 12;
		const password_hash = await bcrypt.hash(newPassword, saltRounds);
		const query = `
      UPDATE users 
      SET password_hash = $1, password_reset_token = NULL, password_reset_expires = NULL -- updated_at handled by trigger
      WHERE id = $2
    `;
		const result = await pool.query(query, [password_hash, id]);
		return result.rowCount > 0;
	}

	// Update password and clear must_change_password flag
	static async updatePasswordAndClearChangeFlag(id, newPassword) {
		const saltRounds = 12;
		const password_hash = await bcrypt.hash(newPassword, saltRounds);
		const query = `
      UPDATE users 
      SET password_hash = $1, 
          must_change_password = FALSE, 
          password_reset_token = NULL, 
          password_reset_expires = NULL -- updated_at handled by trigger
      WHERE id = $2
    `;
		const result = await pool.query(query, [password_hash, id]);
		return result.rowCount > 0;
	}

	// Set password reset token
	static async setPasswordResetToken(email) {
		const token = crypto.randomBytes(32).toString("hex");
		// PostgreSQL TIMESTAMPTZ handles timezone conversion, interval syntax is cleaner
		const expires = new Date(Date.now() + 3600000); // Still use JS Date for interval calculation

		const query = `
      UPDATE users 
      SET password_reset_token = $1, password_reset_expires = $2 -- updated_at handled by trigger
      WHERE email = $3 AND is_active = TRUE
      RETURNING id;
    `;
		const result = await pool.query(query, [token, expires, email]);
		if (result.rowCount > 0) {
			return { token, expires };
		}
		return null;
	}

	// Increment failed login attempts
	static async incrementFailedLoginAttempts(id) {
		const query = `
      UPDATE users 
      SET failed_login_attempts = failed_login_attempts + 1,
          account_locked_until = CASE 
            WHEN failed_login_attempts >= 4 THEN NOW() + interval '30 minutes'
            ELSE account_locked_until
          END -- updated_at handled by trigger
      WHERE id = $1
    `;
		await pool.query(query, [id]);
	}

	// Reset failed login attempts
	static async resetFailedLoginAttempts(id) {
		const query = `
      UPDATE users 
      SET failed_login_attempts = 0, account_locked_until = NULL -- updated_at handled by trigger
      WHERE id = $1
    `;
		await pool.query(query, [id]);
	}

	// Update last login
	static async updateLastLogin(id) {
		const query = "UPDATE users SET last_login = NOW() WHERE id = $1"; // updated_at handled by trigger
		await pool.query(query, [id]);
	}

	// Deactivate user account
	static async deactivateUser(id) {
		const query = "UPDATE users SET is_active = FALSE WHERE id = $1"; // updated_at handled by trigger
		const result = await pool.query(query, [id]);
		return result.rowCount > 0;
	}

	// Activate user account
	static async activateUser(id) {
		const query = "UPDATE users SET is_active = TRUE WHERE id = $1"; // updated_at handled by trigger
		const result = await pool.query(query, [id]);
		return result.rowCount > 0;
	}

	// Update user profile
	static async updateProfile(id, updates) {
		const allowedFields = ["first_name", "last_name", "email"];
		const fieldsToUpdate = [];
		const values = [];
		let paramIndex = 1;

		for (const [key, value] of Object.entries(updates)) {
			if (allowedFields.includes(key) && value !== undefined) {
				// Use double quotes for field names if they might be reserved words, though not strictly necessary here
				fieldsToUpdate.push(`\"${key}\" = $${paramIndex++}`);
				values.push(value);
			}
		}

		if (fieldsToUpdate.length === 0) {
			return false; // No valid fields to update
		}

		values.push(id); // Add id as the last parameter for the WHERE clause

		const query = `
      UPDATE users 
      SET ${fieldsToUpdate.join(", ")} 
      WHERE id = $${paramIndex}
      RETURNING *; 
    `;
		// updated_at is handled by the trigger

		try {
			const result = await pool.query(query, values);
			return result.rows[0]; // Return the updated user object
		} catch (error) {
			if (error.code === "23505" && error.constraint === "users_email_key") {
				throw new Error("Email already exists");
			}
			logger.error("Error updating user profile:", error);
			throw error;
		}
	}

	// Get all users (with pagination and optional filters)
	static async findAll(page = 1, limit = 20, filters = {}) {
		let query =
			"SELECT id, uuid, email, first_name, last_name, role, is_verified, is_active, last_login, created_at, updated_at FROM users";
		const countQuery = "SELECT COUNT(*) FROM users";
		const queryParams = [];
		const filterClauses = [];
		let paramIndex = 1;

		if (filters.role) {
			filterClauses.push(`role = $${paramIndex++}`);
			queryParams.push(filters.role);
		}
		if (filters.is_active !== undefined) {
			filterClauses.push(`is_active = $${paramIndex++}`);
			queryParams.push(filters.is_active);
		}
		// Add more filters as needed

		if (filterClauses.length > 0) {
			query += " WHERE " + filterClauses.join(" AND ");
			// Note: countQuery would also need the WHERE clause. This can get complex.
			// For simplicity, the count query here doesn't include filters. Consider a more robust way if filters are common.
		}

		query += ` ORDER BY created_at DESC LIMIT $${paramIndex++} OFFSET $${paramIndex++}`;
		const offset = (page - 1) * limit;
		queryParams.push(limit, offset);

		const { rows } = await pool.query(query, queryParams);
		// For total count, you might need a separate query without limit/offset but with filters
		// const totalResult = await pool.query(countQuery); // Potentially with filter conditions
		// const total = parseInt(totalResult.rows[0].count, 10);
		// For now, returning rows without total count for simplicity to match old behavior more closely
		return rows;
	}

	// --- User Session Management (Example methods, adjust as needed) ---

	static async saveSession(
		userId,
		tokenJti,
		expiresAt,
		userAgent = null,
		ipAddress = null
	) {
		// expiresAt should be a Date object or a string PostgreSQL can parse
		const query = `
      INSERT INTO user_sessions (user_id, token_jti, expires_at, user_agent, ip_address, created_at)
      VALUES ($1, $2, $3, $4, $5, NOW())
      RETURNING id;
    `;
		const values = [userId, tokenJti, expiresAt, userAgent, ipAddress];
		const result = await pool.query(query, values);
		return result.rows[0].id;
	}

	static async isTokenBlacklisted(tokenJti) {
		const query =
			"SELECT 1 FROM user_sessions WHERE token_jti = $1 AND expires_at > NOW()";
		const { rows } = await pool.query(query, [tokenJti]);
		return rows.length > 0;
	}

	static async revokeSession(tokenJti) {
		// Or, update expires_at to NOW() if you prefer not to delete
		const query = "DELETE FROM user_sessions WHERE token_jti = $1";
		const result = await pool.query(query, [tokenJti]);
		return result.rowCount > 0;
	}

	static async revokeAllUserSessions(userId) {
		const query = "DELETE FROM user_sessions WHERE user_id = $1";
		const result = await pool.query(query, [userId]);
		return result.rowCount > 0;
	}

	static async revokeAllUserSessionsExcept(userId, exceptJti) {
		const query =
			"DELETE FROM user_sessions WHERE user_id = $1 AND token_jti != $2";
		const result = await pool.query(query, [userId, exceptJti]);
		return result.rowCount > 0;
	}

	static async getUserActiveSessions(userId) {
		const query =
			"SELECT id, token_jti, user_agent, ip_address, created_at, expires_at FROM user_sessions WHERE user_id = $1 AND expires_at > NOW() ORDER BY created_at DESC";
		const { rows } = await pool.query(query, [userId]);
		return rows;
	}

	static async getSessionByJti(jti) {
		const query = "SELECT * FROM user_sessions WHERE token_jti = $1";
		const { rows } = await pool.query(query, [jti]);
		return rows[0];
	}

	static async cleanupExpiredSessions() {
		const query = "DELETE FROM user_sessions WHERE expires_at <= NOW()";
		const result = await pool.query(query);
		logger.info(`Cleaned up ${result.rowCount} expired user sessions.`);
		return result.rowCount;
	}

	// --- API Key Management (Example methods, needs hashing for api_key_secret) ---
	static async createApiKey(userId, keyName, daysUntilExpiry = null) {
		const apiKeyIdentifier = crypto.randomBytes(16).toString("hex"); // Public part
		const apiKeySecret = crypto.randomBytes(32).toString("hex"); // Secret part, show ONCE to user

		// IMPORTANT: You MUST hash the apiKeySecret before storing it.
		// Use a strong, one-way hashing algorithm like bcrypt or scrypt.
		// This example will store it directly for simplicity, but DO NOT do this in production.
		const saltRounds = 12; // Or appropriate for your chosen hashing algorithm
		const hashedApiKeySecret = await bcrypt.hash(apiKeySecret, saltRounds);

		let expiresAt = null;
		if (daysUntilExpiry) {
			expiresAt = new Date();
			expiresAt.setDate(expiresAt.getDate() + daysUntilExpiry);
		}

		const query = `
        INSERT INTO api_keys (user_id, key_name, api_key_identifier, hashed_api_key_secret, expires_at, created_at)
        VALUES ($1, $2, $3, $4, $5, NOW())
        RETURNING id, uuid, api_key_identifier, created_at, expires_at;
    `;
		const values = [
			userId,
			keyName,
			apiKeyIdentifier,
			hashedApiKeySecret,
			expiresAt,
		];

		try {
			const result = await pool.query(query, values);
			return { ...result.rows[0], apiKeySecret }; // Return the raw secret ONCE for the user to copy
		} catch (error) {
			if (error.code === "23505") {
				// unique_violation
				// Handle if api_key_identifier somehow collides, though highly unlikely
				logger.error("API Key identifier collision:", error);
			}
			logger.error("Error creating API key:", error);
			throw error;
		}
	}

	static async findApiKeyByIdentifier(identifier) {
		const query = "SELECT * FROM api_keys WHERE api_key_identifier = $1";
		const { rows } = await pool.query(query, [identifier]);
		return rows[0]; // This will include the hashed_api_key_secret
	}

	// Call this after a key is used successfully
	static async updateApiKeyLastUsed(apiKeyId) {
		const query = "UPDATE api_keys SET last_used_at = NOW() WHERE id = $1";
		await pool.query(query, [apiKeyId]);
	}

	static async getUserApiKeys(userId) {
		// Do NOT return hashed_api_key_secret to the user, only metadata
		const query =
			"SELECT id, uuid, user_id, key_name, api_key_identifier, created_at, last_used_at, expires_at FROM api_keys WHERE user_id = $1 ORDER BY created_at DESC";
		const { rows } = await pool.query(query, [userId]);
		return rows;
	}

	static async revokeApiKey(apiKeyId, userId) {
		// Ensure the user owns this API key before revoking
		const query = "DELETE FROM api_keys WHERE id = $1 AND user_id = $2";
		const result = await pool.query(query, [apiKeyId, userId]);
		return result.rowCount > 0;
	}

	// Placeholder for user stats - adjust query as needed for form/submission counts
	static async getUserStats(userId) {
		// This is a simplified example. You'd need to join with forms and submissions tables.
		const query = `
      SELECT 
        (SELECT COUNT(*) FROM forms WHERE user_id = $1) as form_count,
        (SELECT COUNT(*) FROM submissions WHERE user_id = $1) as submission_count 
        -- Add more stats as needed
    `;
		// This query assumes user_id is directly on submissions. Adjust if form_uuid is the link.
		const { rows } = await pool.query(query, [userId]);
		return rows[0] || { form_count: 0, submission_count: 0 };
	}
}

module.exports = User;
