const express = require("express");
const pool = require("../config/database");
const apiAuthMiddleware = require("../middleware/apiAuthMiddleware");
const logger = require("../../config/logger");

const router = express.Router();

// All routes in this file will be protected by API key authentication
router.use(apiAuthMiddleware);

// GET /api/v1/forms - List forms for the authenticated user
router.get("/forms", async (req, res) => {
	try {
		const { rows: forms } = await pool.query(
			`SELECT uuid, name, created_at, is_archived,
       (SELECT COUNT(*) FROM submissions WHERE form_uuid = f.uuid) as submission_count
       FROM forms f 
       WHERE f.user_id = $1 
       ORDER BY f.created_at DESC`,
			[req.user.id] // req.user.id is attached by apiAuthMiddleware
		);
		res.json({ success: true, forms });
	} catch (error) {
		logger.error("API Error fetching forms for user:", {
			userId: req.user.id,
			error,
		});
		res.status(500).json({ success: false, error: "Failed to fetch forms." });
	}
});

// GET /api/v1/forms/:formUuid/submissions - List submissions for a specific form
router.get("/forms/:formUuid/submissions", async (req, res) => {
	const { formUuid } = req.params;
	const page = parseInt(req.query.page) || 1;
	const limit = parseInt(req.query.limit) || 25; // Default 25 submissions per page for API
	const offset = (page - 1) * limit;

	try {
		const { rows: formDetailsRows } = await pool.query(
			"SELECT user_id, name FROM forms WHERE uuid = $1",
			[formUuid]
		);

		if (formDetailsRows.length === 0) {
			return res.status(404).json({ success: false, error: "Form not found." });
		}
		const formDetails = formDetailsRows[0];

		if (formDetails.user_id !== req.user.id) {
			logger.warn(
				`API Access Denied: User ${req.user.id} attempted to access form ${formUuid} owned by ${formDetails.user_id}`
			);
			return res.status(403).json({
				success: false,
				error: "Access denied. You do not own this form.",
			});
		}

		const { rows: countResultRows } = await pool.query(
			"SELECT COUNT(*) as total FROM submissions WHERE form_uuid = $1",
			[formUuid]
		);
		const totalSubmissions = parseInt(countResultRows[0].total, 10);
		const totalPages = Math.ceil(totalSubmissions / limit);

		const { rows: submissions } = await pool.query(
			"SELECT id, data, ip_address, submitted_at FROM submissions WHERE form_uuid = $1 ORDER BY submitted_at DESC LIMIT $2 OFFSET $3",
			[formUuid, limit, offset]
		);

		res.json({
			success: true,
			formName: formDetails.name,
			formUuid,
			pagination: {
				currentPage: page,
				totalPages: totalPages,
				totalSubmissions: totalSubmissions,
				limit: limit,
				perPage: limit, // Alias for limit
				count: submissions.length,
			},
			submissions,
		});
	} catch (error) {
		logger.error("API Error fetching submissions for form:", {
			formUuid,
			userId: req.user.id,
			error,
		});
		res
			.status(500)
			.json({ success: false, error: "Failed to fetch submissions." });
	}
});

module.exports = router;
