const express = require("express");
const pool = require("../config/database"); // pg Pool
const { requireAuth } = require("../middleware/authMiddleware");
const { v4: uuidv4 } = require("uuid"); // Retained for now, though new form UUIDs could be DB generated
const { sendNtfyNotification } = require("../services/notification");
const User = require("../models/User"); // For API Key management
const logger = require("../../config/logger"); // Corrected logger path

const router = express.Router();

// All dashboard routes require authentication
router.use(requireAuth);

// GET /dashboard - Main dashboard view (My Forms)
router.get("/", async (req, res) => {
	try {
		const { rows: forms } = await pool.query(
			`SELECT f.uuid, f.name, f.created_at, f.is_archived,
            (SELECT COUNT(*) FROM submissions WHERE form_uuid = f.uuid) as submission_count
            FROM forms f 
            WHERE f.user_id = $1 
            ORDER BY f.created_at DESC`,
			[req.user.id]
		);

		res.render("dashboard", {
			user: req.user,
			forms: forms,
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "my_forms",
			pageTitle: "My Forms",
		});
	} catch (error) {
		logger.error("Error fetching user forms:", error);
		res.render("dashboard", {
			user: req.user,
			forms: [],
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "my_forms",
			pageTitle: "My Forms",
			error: "Could not load your forms at this time.",
		});
	}
});

// GET /dashboard/create-form - Display page to create a new form
router.get("/create-form", (req, res) => {
	res.render("dashboard", {
		user: req.user,
		appUrl: `${req.protocol}://${req.get("host")}`,
		view: "create_form",
		pageTitle: "Create New Form",
	});
});

// POST /dashboard/forms/create - Handle new form creation
router.post("/forms/create", async (req, res) => {
	const formName = req.body.formName || "Untitled Form";
	// const newUuid = uuidv4(); // UUID will be generated by DB if schema is set up for it
	try {
		// Assuming forms.uuid has DEFAULT gen_random_uuid()
		const {
			rows: [newForm],
		} = await pool.query(
			"INSERT INTO forms (name, user_id) VALUES ($1, $2) RETURNING uuid, name",
			[formName, req.user.id]
		);
		logger.info(
			`Form created: ${newForm.name} with UUID: ${newForm.uuid} for user: ${req.user.id}`
		);

		if (process.env.NTFY_ENABLED === "true" && process.env.NTFY_TOPIC_URL) {
			try {
				await sendNtfyNotification(
					"New Form Created (User)",
					`Form \"${newForm.name}\" (UUID: ${newForm.uuid}) was created by user ${req.user.email}.`,
					"high"
				);
			} catch (ntfyError) {
				logger.error(
					"Failed to send ntfy notification for new form creation:",
					ntfyError
				);
			}
		}

		res.redirect("/dashboard");
	} catch (error) {
		logger.error("Error creating form for user:", error);
		res.render("dashboard", {
			user: req.user,
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "create_form",
			pageTitle: "Create New Form",
			error: "Failed to create form. Please try again.",
			formNameValue: formName,
		});
	}
});

// GET /dashboard/submissions/:formUuid - View submissions for a specific form
router.get("/submissions/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const page = parseInt(req.query.page) || 1;
	const limit = parseInt(req.query.limit) || 10;
	const offset = (page - 1) * limit;

	try {
		const { rows: formDetailsRows } = await pool.query(
			"SELECT name, user_id FROM forms WHERE uuid = $1",
			[formUuid]
		);

		if (formDetailsRows.length === 0) {
			return res.render("dashboard", {
				user: req.user,
				view: "my_forms",
				pageTitle: "Form Not Found",
				error: "The form you are looking for does not exist.",
				appUrl: `${req.protocol}://${req.get("host")}`,
				forms: [],
			});
		}
		const formDetails = formDetailsRows[0];

		if (formDetails.user_id !== req.user.id) {
			return res.render("dashboard", {
				user: req.user,
				view: "my_forms",
				pageTitle: "Access Denied",
				error: "You do not have permission to view submissions for this form.",
				appUrl: `${req.protocol}://${req.get("host")}`,
				forms: [],
			});
		}

		const formName = formDetails.name;

		const { rows: countResultRows } = await pool.query(
			"SELECT COUNT(*) as total FROM submissions WHERE form_uuid = $1",
			[formUuid]
		);
		const totalSubmissions = parseInt(countResultRows[0].total, 10);
		const totalPages = Math.ceil(totalSubmissions / limit);

		const { rows: submissions } = await pool.query(
			"SELECT id, data, ip_address, submitted_at FROM submissions WHERE form_uuid = $1 ORDER BY submitted_at DESC LIMIT $2 OFFSET $3",
			[formUuid, limit, offset]
		);

		res.render("dashboard", {
			user: req.user,
			view: "form_submissions",
			pageTitle: `Submissions for ${formName}`,
			submissions: submissions,
			formUuid: formUuid,
			formName: formName,
			appUrl: `${req.protocol}://${req.get("host")}`,
			pagination: {
				currentPage: page,
				totalPages: totalPages,
				totalSubmissions: totalSubmissions,
				limit: limit,
			},
		});
	} catch (error) {
		logger.error(
			"Error fetching submissions for form:",
			formUuid,
			"user:",
			req.user.id,
			error
		);
		res.render("dashboard", {
			user: req.user,
			view: "form_submissions",
			pageTitle: "Error Loading Submissions",
			error:
				"Could not load submissions for this form. Please try again later.",
			formUuid: formUuid,
			formName: "Error",
			submissions: [],
			appUrl: `${req.protocol}://${req.get("host")}`,
			pagination: {
				currentPage: 1,
				totalPages: 1,
				totalSubmissions: 0,
				limit: limit,
			},
		});
	}
});

// GET /dashboard/submissions/:formUuid/export - Export submissions to CSV
router.get("/submissions/:formUuid/export", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const { rows: formDetailsRows } = await pool.query(
			"SELECT name, user_id FROM forms WHERE uuid = $1",
			[formUuid]
		);

		if (formDetailsRows.length === 0) {
			return res.status(404).send("Form not found.");
		}
		const formDetails = formDetailsRows[0];

		if (formDetails.user_id !== req.user.id) {
			return res.status(403).send("Access denied. You do not own this form.");
		}
		const formName = formDetails.name;

		const { rows: submissions } = await pool.query(
			"SELECT data, ip_address, submitted_at FROM submissions WHERE form_uuid = $1 ORDER BY submitted_at DESC",
			[formUuid]
		);

		// Create CSV content
		let headers = ["Submitted At", "IP Address"]; // Initialize with default headers
		const dataRows = submissions.map((submission) => {
			// Ensure submission.data is parsed if it's a JSON string, or used directly if already an object
			let data = {};
			if (typeof submission.data === "string") {
				try {
					data = JSON.parse(submission.data);
				} catch (e) {
					logger.warn(
						`Failed to parse submission data for form ${formUuid}, submission ID ${submission.id}: ${submission.data}`
					);
					// Potentially include raw data or an error placeholder
					data = { error_parsing_data: submission.data };
				}
			} else if (
				typeof submission.data === "object" &&
				submission.data !== null
			) {
				data = submission.data;
			} else {
				logger.warn(
					`Unexpected submission data format for form ${formUuid}, submission ID ${submission.id}:`,
					submission.data
				);
				data = { unexpected_data_format: String(submission.data) };
			}

			// Dynamically add keys from parsed data to headers, ensuring no duplicates
			Object.keys(data).forEach((key) => {
				if (!headers.includes(key)) {
					headers.push(key);
				}
			});
			return {
				"Submitted At": new Date(submission.submitted_at).toISOString(),
				"IP Address": submission.ip_address,
				...data,
			};
		});

		let csvContent = headers.join(",") + "\n";
		dataRows.forEach((row) => {
			const values = headers.map((header) => {
				const value =
					row[header] === null || row[header] === undefined ? "" : row[header];
				return `"${String(value).replace(/"/g, '""')}"`;
			});
			csvContent += values.join(",") + "\n";
		});

		res.header("Content-Type", "text/csv");
		res.attachment(
			`submissions-${formName.replace(/\s+/g, "_")}-${formUuid}.csv`
		);
		res.send(csvContent);
	} catch (error) {
		logger.error("Error exporting submissions:", error);
		res.status(500).send("Error exporting submissions.");
	}
});

// GET /dashboard/forms/:formUuid/settings - Display form settings page
router.get("/forms/:formUuid/settings", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const { rows: formRows } = await pool.query(
			"SELECT * FROM forms WHERE uuid = $1 AND user_id = $2",
			[formUuid, req.user.id]
		);
		if (formRows.length === 0) {
			return res.status(404).render("dashboard", {
				user: req.user,
				view: "my_forms",
				pageTitle: "Not Found",
				error: "Form not found or you do not have permission to access it.",
				appUrl: `${req.protocol}://${req.get("host")}`,
				forms: [],
			});
		}
		res.render("dashboard", {
			user: req.user,
			form: formRows[0],
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "form_settings",
			pageTitle: `Settings for ${formRows[0].name}`,
		});
	} catch (error) {
		logger.error("Error fetching form settings:", error);
		res.status(500).render("dashboard", {
			user: req.user,
			view: "my_forms",
			pageTitle: "Error",
			error: "Error fetching form settings.",
			appUrl: `${req.protocol}://${req.get("host")}`,
			forms: [],
		});
	}
});

// POST /dashboard/forms/:formUuid/settings - Update form settings
router.post("/forms/:formUuid/settings", async (req, res) => {
	const { formUuid } = req.params;
	const {
		formName,
		thankYouUrl,
		thankYouMessage,
		ntfyEnabled,
		allowedDomains,
		emailNotificationsEnabled,
		notificationEmailAddress,
		recaptchaEnabled,
	} = req.body;

	try {
		// Verify user owns the form first
		const { rows: formCheckRows } = await pool.query(
			"SELECT user_id FROM forms WHERE uuid = $1",
			[formUuid]
		);
		if (
			formCheckRows.length === 0 ||
			formCheckRows[0].user_id !== req.user.id
		) {
			return res.status(403).send("Access denied or form not found.");
		}

		// Construct the update query dynamically to only update provided fields
		const updates = [];
		const values = [];
		let paramIndex = 1;

		if (formName !== undefined) {
			updates.push(`name = $${paramIndex++}`);
			values.push(formName);
		}
		if (thankYouUrl !== undefined) {
			updates.push(`thank_you_url = $${paramIndex++}`);
			values.push(thankYouUrl || null);
		}
		if (thankYouMessage !== undefined) {
			updates.push(`thank_you_message = $${paramIndex++}`);
			values.push(thankYouMessage || null);
		}
		if (ntfyEnabled !== undefined) {
			updates.push(`ntfy_enabled = $${paramIndex++}`);
			values.push(ntfyEnabled === "on" || ntfyEnabled === true);
		}
		if (allowedDomains !== undefined) {
			updates.push(`allowed_domains = $${paramIndex++}`);
			values.push(allowedDomains || null);
		}
		if (emailNotificationsEnabled !== undefined) {
			updates.push(`email_notifications_enabled = $${paramIndex++}`);
			values.push(
				emailNotificationsEnabled === "on" || emailNotificationsEnabled === true
			);
		}
		if (notificationEmailAddress !== undefined) {
			updates.push(`notification_email_address = $${paramIndex++}`);
			values.push(notificationEmailAddress || null);
		}
		if (recaptchaEnabled !== undefined) {
			updates.push(`recaptcha_enabled = $${paramIndex++}`);
			values.push(recaptchaEnabled === "on" || recaptchaEnabled === true);
		}

		if (updates.length === 0) {
			return res.redirect(
				`/dashboard/forms/${formUuid}/settings?success=No changes detected`
			);
		}

		values.push(formUuid);
		values.push(req.user.id);

		const query = `UPDATE forms SET ${updates.join(", ")}, updated_at = NOW() WHERE uuid = $${paramIndex++} AND user_id = $${paramIndex++}`;

		const { rowCount } = await pool.query(query, values);

		if (rowCount > 0) {
			res.redirect(
				`/dashboard/forms/${formUuid}/settings?success=Form settings updated successfully.`
			);
		} else {
			// This case should ideally not happen if the form ownership check passed
			res.redirect(
				`/dashboard/forms/${formUuid}/settings?error=Failed to update settings or no changes made.`
			);
		}
	} catch (error) {
		logger.error("Error updating form settings:", error);
		res.redirect(
			`/dashboard/forms/${formUuid}/settings?error=Error updating form settings.`
		);
	}
});

// POST /dashboard/forms/:formUuid/archive - Archive a form
router.post("/forms/:formUuid/archive", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const { rowCount } = await pool.query(
			"UPDATE forms SET is_archived = TRUE, updated_at = NOW() WHERE uuid = $1 AND user_id = $2",
			[formUuid, req.user.id]
		);
		if (rowCount > 0) {
			res.redirect("/dashboard?archived=true");
		} else {
			res.status(404).send("Form not found or not owned by user.");
		}
	} catch (error) {
		logger.error("Error archiving form:", error);
		res.status(500).send("Error archiving form.");
	}
});

// POST /dashboard/forms/:formUuid/unarchive - Unarchive a form
router.post("/forms/:formUuid/unarchive", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const { rowCount } = await pool.query(
			"UPDATE forms SET is_archived = FALSE, updated_at = NOW() WHERE uuid = $1 AND user_id = $2",
			[formUuid, req.user.id]
		);
		if (rowCount > 0) {
			res.redirect("/dashboard?unarchived=true");
		} else {
			res.status(404).send("Form not found or not owned by user.");
		}
	} catch (error) {
		logger.error("Error unarchiving form:", error);
		res.status(500).send("Error unarchiving form.");
	}
});

// POST /dashboard/forms/:formUuid/delete - Delete a form
router.post("/forms/:formUuid/delete", async (req, res) => {
	const { formUuid } = req.params;
	try {
		// Add additional checks or soft delete if needed
		const { rowCount } = await pool.query(
			"DELETE FROM forms WHERE uuid = $1 AND user_id = $2",
			[formUuid, req.user.id]
		);
		if (rowCount > 0) {
			res.redirect("/dashboard?deleted=true");
		} else {
			res.status(404).send("Form not found or not owned by user.");
		}
	} catch (error) {
		logger.error("Error deleting form:", error);
		res.status(500).send("Error deleting form.");
	}
});

// GET /dashboard/profile - Display user profile page
router.get("/profile", async (req, res) => {
	try {
		// Fetch the full user object for the profile page, could use User model
		const userProfile = await User.findById(req.user.id);
		if (!userProfile) {
			logger.warn(
				`User not found in DB for ID: ${req.user.id} during profile view`
			);
			return res.status(404).send("User profile not found.");
		}
		res.render("dashboard", {
			user: userProfile, // Pass the full userProfile object
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "profile_settings",
			pageTitle: "My Profile",
		});
	} catch (error) {
		logger.error("Error fetching user profile:", error);
		res.status(500).render("dashboard", {
			user: req.user, // Fallback to req.user if profile fetch fails
			view: "profile_settings",
			pageTitle: "My Profile",
			error: "Could not load your profile information.",
			appUrl: `${req.protocol}://${req.get("host")}`,
		});
	}
});

// POST /dashboard/profile - Update user profile
router.post("/profile", async (req, res) => {
	const { firstName, lastName, email } = req.body;
	try {
		const updatedUser = await User.updateProfile(req.user.id, {
			first_name: firstName,
			last_name: lastName,
			email: email,
		});

		if (updatedUser) {
			// Update the session user object if email changes, etc.
			// This is important because req.user is populated from the session at login.
			// If email is part of the identifier or used for display, it needs to be fresh.
			req.login(updatedUser, (err) => {
				// req.login is from Passport to update session user
				if (err) {
					logger.error("Error updating session after profile update:", err);
					return res.redirect("/dashboard/profile?error=Session update failed");
				}
				return res.redirect(
					"/dashboard/profile?success=Profile updated successfully"
				);
			});
		} else {
			res.redirect(
				"/dashboard/profile?error=Failed to update profile or no changes made"
			);
		}
	} catch (error) {
		logger.error("Error updating profile:", error);
		let errorMessage = "Error updating profile.";
		if (error.message === "Email already exists") {
			errorMessage =
				"That email address is already in use. Please choose another.";
		}
		res.redirect(
			`/dashboard/profile?error=${encodeURIComponent(errorMessage)}`
		);
	}
});

// POST /dashboard/profile/change-password - Change user password
router.post("/profile/change-password", async (req, res) => {
	const { currentPassword, newPassword, confirmPassword } = req.body;

	if (newPassword !== confirmPassword) {
		return res.redirect(
			"/dashboard/profile?passError=New passwords do not match."
		);
	}
	if (!newPassword || newPassword.length < 8) {
		// Basic validation
		return res.redirect(
			"/dashboard/profile?passError=New password must be at least 8 characters long."
		);
	}

	try {
		const user = await User.findById(req.user.id);
		if (!user) {
			return res.status(400).send("User not found.");
		}

		const isMatch = await bcrypt.compare(currentPassword, user.password_hash);
		if (!isMatch) {
			return res.redirect(
				"/dashboard/profile?passError=Incorrect current password."
			);
		}

		const success = await User.updatePassword(req.user.id, newPassword);
		if (success) {
			// Optionally, log out other sessions for security
			// await User.revokeAllUserSessionsExcept(req.user.id, req.session.jwtJti); // Assuming jwtJti is stored in session
			res.redirect(
				"/dashboard/profile?passSuccess=Password changed successfully."
			);
		} else {
			res.redirect("/dashboard/profile?passError=Failed to change password.");
		}
	} catch (error) {
		logger.error("Error changing password:", error);
		res.redirect("/dashboard/profile?passError=Error changing password.");
	}
});

// API Keys Section
// GET /dashboard/api-keys - Display API keys page
router.get("/api-keys", async (req, res) => {
	try {
		const apiKeys = await User.getUserApiKeys(req.user.id);
		res.render("dashboard", {
			user: req.user,
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "api_keys",
			pageTitle: "API Keys",
			apiKeys: apiKeys,
			newApiKey: req.query.newApiKey, // For showing the new key once
			newApiKeyName: req.query.newApiKeyName,
		});
	} catch (error) {
		logger.error("Error fetching API keys:", error);
		res.render("dashboard", {
			user: req.user,
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "api_keys",
			pageTitle: "API Keys",
			error: "Could not load your API keys.",
			apiKeys: [],
		});
	}
});

// POST /dashboard/api-keys/create - Create a new API key
router.post("/api-keys/create", async (req, res) => {
	const { keyName } = req.body;
	if (!keyName || keyName.trim() === "") {
		return res.redirect(
			"/dashboard/api-keys?keyError=API Key name cannot be empty."
		);
	}
	try {
		// User.createApiKey should handle hashing and return the raw secret ONCE
		const { apiKeySecret, ...newKeyDetails } = await User.createApiKey(
			req.user.id,
			keyName.trim()
		);

		// Pass the raw secret key to the view via query param for the user to copy ONCE.
		// This is a common pattern but ensure it's clear this is the only time it's shown.
		res.redirect(
			`/dashboard/api-keys?success=API Key created successfully.&newApiKey=${encodeURIComponent(apiKeySecret)}&newApiKeyName=${encodeURIComponent(newKeyDetails.key_name)}`
		);
	} catch (error) {
		logger.error("Error creating API key:", error);
		res.redirect("/dashboard/api-keys?keyError=Error creating API key.");
	}
});

// POST /dashboard/api-keys/:apiKeyId/delete - Delete an API key
router.post("/api-keys/:apiKeyId/delete", async (req, res) => {
	const { apiKeyId } = req.params;
	try {
		const success = await User.revokeApiKey(apiKeyId, req.user.id);
		if (success) {
			res.redirect("/dashboard/api-keys?deleted=API Key deleted successfully.");
		} else {
			res.redirect(
				"/dashboard/api-keys?keyError=Failed to delete API Key or key not found."
			);
		}
	} catch (error) {
		logger.error("Error deleting API key:", error);
		res.redirect("/dashboard/api-keys?keyError=Error deleting API key.");
	}
});

// GET /dashboard/settings - Main settings page (could link to profile, api keys, etc.)
router.get("/settings", (req, res) => {
	res.render("dashboard", {
		user: req.user,
		appUrl: `${req.protocol}://${req.get("host")}`,
		view: "general_settings", // A new EJS view or section for general settings
		pageTitle: "Settings",
	});
});

module.exports = router;
