const express = require("express");
const pool = require("../config/database"); // pg Pool
const { sendNtfyNotification } = require("../services/notification");
const { sendSubmissionNotification } = require("../services/emailService");
const { verifyRecaptchaV2 } = require("../utils/recaptchaHelper");
const {
	createSubmissionRateLimiter,
	createFormSpecificRateLimiter,
	createStrictRateLimiter,
} = require("../middleware/redisRateLimiter");
const domainChecker = require("../middleware/domainChecker");
const logger = require("../../config/logger"); // Corrected logger path

const router = express.Router();

// Initialize rate limiters
const submissionRateLimit = createSubmissionRateLimiter();
const formSpecificRateLimit = createFormSpecificRateLimiter();
const strictRateLimit = createStrictRateLimiter();

router.get("/health", (req, res) => res.status(200).json({ status: "ok" }));

// Render login page
router.get("/login", (req, res) => {
	res.render("login", {
		error: req.query.error,
		success: req.query.success,
		email: req.query.email,
	});
});

// Render registration page
router.get("/register", (req, res) => {
	res.render("register", {
		error: req.query.error,
		success: req.query.success,
		email: req.query.email,
		first_name: req.query.first_name,
		last_name: req.query.last_name,
	});
});

router.post(
	"/submit/:formUuid",
	strictRateLimit,
	submissionRateLimit,
	formSpecificRateLimit,
	domainChecker,
	async (req, res) => {
		const { formUuid } = req.params;
		const submissionData = { ...req.body };
		const ipAddress = req.ip;

		const recaptchaToken = submissionData["g-recaptcha-response"];
		delete submissionData["g-recaptcha-response"];

		if (submissionData.honeypot_field && submissionData.honeypot_field !== "") {
			logger.info(
				`Honeypot triggered for ${formUuid} by IP ${ipAddress}. Ignoring submission.`
			);
			if (submissionData._thankyou) {
				return res.redirect(submissionData._thankyou);
			}
			return res.send(
				"<h1>Thank You!</h1><p>Your submission has been received.</p>"
			);
		}

		let formSettings;
		try {
			const { rows: forms } = await pool.query(
				"SELECT id, user_id, name, thank_you_url, thank_you_message, ntfy_enabled, is_archived, email_notifications_enabled, notification_email_address, recaptcha_enabled FROM forms WHERE uuid = $1",
				[formUuid]
			);
			if (forms.length === 0) {
				logger.warn(
					`Submission attempt to non-existent form UUID: ${formUuid} from IP: ${ipAddress}`
				);
				return res.status(404).send("Form endpoint not found.");
			}
			formSettings = forms[0];

			if (formSettings.is_archived) {
				logger.warn(
					`Submission attempt to archived form UUID: ${formUuid} from IP: ${ipAddress}`
				);
				return res
					.status(410)
					.send(
						"This form has been archived and is no longer accepting submissions."
					);
			}
		} catch (dbError) {
			logger.error("Error fetching form settings during submission:", {
				formUuid,
				error: dbError,
			});
			return res
				.status(500)
				.send("Error processing submission due to a configuration issue."); // More generic error to user
		}

		if (formSettings.recaptcha_enabled) {
			if (!recaptchaToken) {
				logger.warn(
					`reCAPTCHA enabled for form ${formUuid} but no token provided by IP ${ipAddress}.`
				);
				return res
					.status(403)
					.send(
						"reCAPTCHA is required for this form. Please complete the challenge."
					);
			}

			const isRecaptchaValid = await verifyRecaptchaV2(
				recaptchaToken,
				ipAddress
			);
			if (!isRecaptchaValid) {
				logger.warn(
					`reCAPTCHA verification failed for form ${formUuid} from IP ${ipAddress}.`
				);
				return res
					.status(403)
					.send("reCAPTCHA verification failed. Please try again.");
			}
		}

		let formNameForNotification = formSettings.name || `Form ${formUuid}`;
		try {
			const ntfyEnabled = formSettings.ntfy_enabled;
			const formOwnerUserId = formSettings.user_id; // This should be NOT NULL based on forms schema

			const formForEmail = {
				name: formSettings.name,
				email_notifications_enabled: formSettings.email_notifications_enabled,
				notification_email_address: formSettings.notification_email_address,
			};

			let ownerEmail = null;
			if (formOwnerUserId) {
				// Should always be true if form exists
				const { rows: users } = await pool.query(
					"SELECT email FROM users WHERE id = $1",
					[formOwnerUserId]
				);
				if (users.length > 0) {
					ownerEmail = users[0].email;
				} else {
					logger.warn(
						`Owner user with ID ${formOwnerUserId} not found for form ${formUuid}, though form record exists.`
					);
				}
			}

			// The user_id in submissions table is NOT NULL in PostgreSQL schema, ensure formOwnerUserId is valid.
			if (!formOwnerUserId) {
				logger.error(
					`Critical: formOwnerUserId is null for form ${formUuid} during submission. This should not happen if form exists.`
				);
				// Potentially send an alert to admin here
				return res
					.status(500)
					.send("Error processing submission due to inconsistent data.");
			}

			await pool.query(
				"INSERT INTO submissions (form_uuid, user_id, data, ip_address) VALUES ($1, $2, $3, $4)",
				[formUuid, formOwnerUserId, JSON.stringify(submissionData), ipAddress]
			);
			logger.info(
				`Submission received for ${formUuid} (user: ${formOwnerUserId}): ${JSON.stringify(submissionData)}`
			);

			const submissionSummary = Object.entries(submissionData)
				.filter(([key]) => key !== "_thankyou") // Ensure _thankyou is not in summary
				.map(([key, value]) => `${key}: ${value}`)
				.join(", ");

			if (ntfyEnabled) {
				sendNtfyNotification(
					`New Submission: ${formNameForNotification}`,
					`Data: ${
						submissionSummary || "No data fields"
					}\nFrom IP: ${ipAddress}`,
					"high",
					"incoming_form"
				).catch((err) =>
					logger.error("Failed to send NTFY notification:", err)
				); // Log & continue
			}

			if (ownerEmail) {
				sendSubmissionNotification(
					formForEmail,
					submissionData,
					ownerEmail
				).catch((err) =>
					logger.error("Failed to send submission email:", {
						formUuid,
						recipient: ownerEmail,
						error: err,
					})
				);
			} else if (
				formForEmail.email_notifications_enabled &&
				!formForEmail.notification_email_address
			) {
				logger.warn(
					`Email notification enabled for form ${formUuid} but owner email could not be determined and no custom address set.`
				);
			}

			if (formSettings.thank_you_url) {
				return res.redirect(formSettings.thank_you_url);
			}

			if (formSettings.thank_you_message) {
				const safeMessage = formSettings.thank_you_message
					.replace(/&/g, "&amp;")
					.replace(/</g, "&lt;")
					.replace(/>/g, "&gt;")
					.replace(/"/g, "&quot;")
					.replace(/'/g, "&#039;");
				return res.send(safeMessage);
			}

			if (submissionData._thankyou) {
				return res.redirect(submissionData._thankyou);
			}

			res.send(
				'<h1>Thank You!</h1><p>Your submission has been received.</p><p><a href="/">Back to form manager</a></p>'
			);
		} catch (error) {
			logger.error("Error processing submission (main block):", {
				formUuid,
				error: error.message,
				stack: error.stack,
			});
			// Avoid sending detailed error to client, but log it.
			sendNtfyNotification(
				`Submission Error: ${formNameForNotification}`,
				`Failed to process submission for ${formUuid} from IP ${ipAddress}. Error: ${error.message}`,
				"max"
			).catch((err) =>
				logger.error("Failed to send error NTFY notification:", err)
			);

			res
				.status(500)
				.send(
					"An error occurred while processing your submission. Please try again later."
				);
		}
	}
);

module.exports = router;
