const crypto = require("crypto");
const bcrypt = require("bcryptjs");

const API_KEY_IDENTIFIER_PREFIX = "fsk"; // Formies Secret Key
const API_KEY_IDENTIFIER_LENGTH = 12; // Length of the random part of the identifier
const API_KEY_SECRET_LENGTH = 32; // Length of the secret part in bytes, results in 2x hex string length

/**
 * Generates a new API key parts: the full key (to show to user once) and its components for storage.
 * Identifier: A public, non-secret unique string for lookup (e.g., 'fsk_abcdef123').
 * Secret: A cryptographically strong random string.
 * Full Key: Identifier + '_' + Secret (this is what the user gets).
 * @returns {{ fullApiKey: string, identifier: string, secret: string }}
 */
function generateApiKeyParts() {
	const randomIdentifierPart = crypto
		.randomBytes(Math.ceil(API_KEY_IDENTIFIER_LENGTH / 2))
		.toString("hex")
		.slice(0, API_KEY_IDENTIFIER_LENGTH);
	const identifier = `${API_KEY_IDENTIFIER_PREFIX}_${randomIdentifierPart}`;
	const secret = crypto.randomBytes(API_KEY_SECRET_LENGTH).toString("hex");
	const fullApiKey = `${identifier}_${secret}`;
	return { fullApiKey, identifier, secret };
}

/**
 * Hashes an API key secret using bcrypt.
 * @param {string} apiKeySecret - The secret part of the API key.
 * @returns {Promise<string>} - The hashed API key secret.
 */
async function hashApiKeySecret(apiKeySecret) {
	const saltRounds = 10; // Standard practice
	return bcrypt.hash(apiKeySecret, saltRounds);
}

/**
 * Compares a plain text API key secret with a stored hashed secret.
 * @param {string} plainTextSecret - The plain text secret part provided by the user.
 * @param {string} hashedSecret - The stored hashed secret from the database.
 * @returns {Promise<boolean>} - True if the secrets match, false otherwise.
 */
async function compareApiKeySecret(plainTextSecret, hashedSecret) {
	return bcrypt.compare(plainTextSecret, hashedSecret);
}

module.exports = {
	generateApiKeyParts,
	hashApiKeySecret,
	compareApiKeySecret,
	API_KEY_IDENTIFIER_PREFIX,
};
